@@ -31,7 +31,6 @@
 #include <pthread.h>
 #include <signal.h>
 
-#include "loadcat.h"
 #include "addisk.h"
 #include "config_common.h"
 #include "fastaddisk.h"
@@ -116,7 +115,7 @@ guint du_s (gchar * path)
 	  /* FIXME: STDIN should be reopened to the real STDIN. */
 	}
     }
-  return (atoi (buffer)*1024);
+  return (atoi (buffer) * 1024);
 }
 
 gboolean
@@ -212,7 +211,22 @@ umount_disk (CONFIG * my_config)
   return (TRUE);
 }
 
+void
+umount_and_eject_if_necessary ()
+{
+  if (flag_to_umount == TRUE)
+    {
+      /* umount the drive and eject it */
+      if (my_config->eject_disk == TRUE)
+	{
+	  if (my_config->use_supermount == TRUE)
+	    eject_disk (my_config);
+	  else if (umount_disk (my_config) == TRUE)
+	    eject_disk (my_config);
+	}
+    }
 
+}
 gint get_thread_status ()
 {
   return (_thread_status);
@@ -270,20 +284,9 @@ my_timeout (gpointer data)
 	}
       g_free (local_diskname);
       g_free (local_foldername);
-      if (flag_to_umount == TRUE)
-	{
-	  /* umount the drive and eject it */
-	  if ((my_config->use_supermount == FALSE) && (umount_disk (my_config) == FALSE))
-	    {
-	      /* Error */
-	      ERROR_DIALOG ("Unable to umount your cdrom.\nDisk in use ?",
-			    main_window);
-	    } else {
-	  if (my_config->eject_disk == TRUE)
-	    {
-	      eject_disk (my_config);
-	    }
-	} }
+/************** If umount and eject have to be in the main thread, remove the comments ********/
+umount_and_eject_if_necessary();
+/*********************************************************************************************/
       reinit_thread_status ();
     }
   else
@@ -513,9 +516,9 @@ start_thread_scan (void *arg)
   create_folder_tree (racine, local_gn, local_foldername,
 		      my_config->getInformation);
 
+//  umount_and_eject_if_necessary ();
   set_thread_status_to_ended ();
   racine->is_modified = TRUE;
   pthread_exit (NULL);
   return (NULL);
 }
-
