@@ -112,6 +112,8 @@ fad_create_window (gchar ** fast_name)
       free (*fast_name);
       *fast_name = strdup (temp);
 
+      /* FIXME: And what if fast_name is empty or already exists ? */
+
       gnome_dialog_close (GNOME_DIALOG (name_window));
       return 1;
     }
@@ -127,11 +129,11 @@ fast_add_disk (FOLDER * racine)
 {
   gint name_window_status;
 
+
   /* Mount if necessary */
   if ((my_config->use_supermount == FALSE) && (mount_disk (my_config) == FALSE))
     {
       /* Error */
-      ERROR_DIALOG ("Unable to mount your cdrom.\nNo Disk ?", main_window);
       return;
     }
 
@@ -165,6 +167,7 @@ fast_add_disk (FOLDER * racine)
       if (name_window_status == 1)
 	{
 	  set_local_disk_name (my_config->fast_name);
+	  set_added_disk (TRUE);
 	}
       else
 	{
