@@ -18,30 +18,51 @@
 
 #include <config.h>
 #include <gnome.h>
-#include <stdio.h>
-#include <stdlib.h>
+#include <sys/types.h>
+#include <sys/stat.h>
 #include <unistd.h>
-#include <errno.h>
-#include <sys/wait.h>
-#include <fnmatch.h>
+#include <fcntl.h>
+#include <zlib.h>
 
 #include "folder.h"
 #include "loadcat.h"
 
-static FILE *catalog;
-
 GString *
-fread_gstring (FILE * fh)
+read_gstring (int fh)
 {
   gint l;
   GString *r;
   gchar *s;
 
-  fread (&l, sizeof (gint), 1, fh);
+  read (fh, &l, sizeof (gint));
   if (l)
     {
-      s = (gchar *) g_malloc (sizeof (gchar *) * (l + 1));
-      fread (s, sizeof (gchar), l, fh);
+      s = (gchar *) g_malloc (sizeof (gchar) * (l + 1));
+      read (fh, s, sizeof (gchar) * l);
+      s[l] = 0;
+      r = g_string_new (s);
+      free (s);
+    }
+  else
+    {
+      r = NULL;
+    }
+
+  return (r);
+}
+
+GString *
+gzread_gstring (gzFile gzfh)
+{
+  gint l;
+  GString *r;
+  gchar *s;
+
+  gzread (gzfh, &l, sizeof (gint));
+  if (l)
+    {
+      s = (gchar *) g_malloc (sizeof (gchar) * (l + 1));
+      gzread (gzfh, s, sizeof (gchar) * l);
       s[l] = 0;
       r = g_string_new (s);
       free (s);
@@ -55,22 +76,36 @@ fread_gstring (FILE * fh)
 }
 
 FILE_DATA *
-fread_filedata (FILE * fh)
+read_filedata (gzFile gzfh, int fh)
 {
   FILE_DATA *d;
   d = (FILE_DATA *) g_malloc (sizeof (FILE_DATA));
-  d->name = fread_gstring (fh);
-  fread (&(d->taille), sizeof (guint32), 1, fh);
-  fread (&(d->type), sizeof (guint), 1, fh);
-  fread (&(d->date), sizeof (time_t), 1, fh);
-  fread (&(d->categorie), sizeof (guint16), 1, fh);
-  fread (&(d->description), sizeof (guint16), 1, fh);
-  d->information = fread_gstring (fh);
+  if (fh >= 0)
+    {
+      d->name = read_gstring (fh);
+      read (fh, &(d->taille), sizeof (guint32));
+      read (fh, &(d->type), sizeof (guint));
+      read (fh, &(d->date), sizeof (time_t));
+      read (fh, &(d->categorie), sizeof (guint16));
+      read (fh, &(d->description), sizeof (guint16));
+      d->information = read_gstring (fh);
+
+    }
+  else
+    {
+      d->name = gzread_gstring (gzfh);
+      gzread (gzfh, &(d->taille), sizeof (guint32));
+      gzread (gzfh, &(d->type), sizeof (guint));
+      gzread (gzfh, &(d->date), sizeof (time_t));
+      gzread (gzfh, &(d->categorie), sizeof (guint16));
+      gzread (gzfh, &(d->description), sizeof (guint16));
+      d->information = gzread_gstring (gzfh);
+    }
   return (d);
 }
 
 void
-load_folder_tree (FOLDER * racine, FILE * fh)
+load_folder_tree (FOLDER * racine, gzFile gzcatalog, int catalog)
 {
   gint i = 1;
   gchar id = CAT_TREE_0;
@@ -79,7 +114,10 @@ load_folder_tree (FOLDER * racine, FILE * fh)
 
   prev_node = racine->tree;
 
-  fread (&id, sizeof (gchar), 1, fh);
+  if (catalog >= 0)
+    read (catalog, &id, sizeof (gchar));
+  else
+    gzread (gzcatalog, &id, sizeof (gchar));
   while (id != CAT_TREE_END)
     {
       switch (id)
@@ -87,7 +125,7 @@ load_folder_tree (FOLDER * racine, FILE * fh)
 	case CAT_TREE_0:
 	  prev_node =
 	    g_node_append_data (prev_node->parent, GUINT_TO_POINTER (i++));
-	  fd = fread_filedata (fh);
+	  fd = read_filedata (gzcatalog, catalog);
 	  fd->node = prev_node;
 	  g_ptr_array_add (racine->datas, fd);
 	  break;
@@ -95,104 +133,57 @@ load_folder_tree (FOLDER * racine, FILE * fh)
 	  prev_node = prev_node->parent;
 	  break;
 	case CAT_TREE_DOWN:
-	  fread (&id, sizeof (gchar), 1, fh);
+	  if (catalog >= 0)
+	    read (catalog, &id, sizeof (gchar));
+	  else
+	    gzread (gzcatalog, &id, sizeof (gchar));
 	  if (id != CAT_TREE_UP)
 	    {
 	      g_assert (id == CAT_TREE_0);
 	      prev_node =
 		g_node_append_data (prev_node, GUINT_TO_POINTER (i++));
-	      fd = fread_filedata (fh);
+	      fd = read_filedata (gzcatalog, catalog);
 	      fd->node = prev_node;
 	      g_ptr_array_add (racine->datas, fd);
 	    }
 	  break;
 	}
-      fread (&id, sizeof (gchar), 1, fh);
+      if (catalog >= 0)
+	read (catalog, &id, sizeof (gchar));
+      else
+	gzread (gzcatalog, &id, sizeof (gchar));
     }
 }
 
 GPtrArray *
-load_gptrarray_of_gstrings (FILE * fh)
+load_gptrarray_of_gstrings (gzFile gzfh, int fh)
 {
   GPtrArray *gpa;
   gint i;
   gint nb_datas = 0;
 
   gpa = g_ptr_array_new ();
-  fread (&nb_datas, sizeof (gint), 1, fh);
+  if (fh >= 0)
+    read (fh, &nb_datas, sizeof (gint));
+  else
+    gzread (gzfh, &nb_datas, sizeof (gint));
   for (i = 0; i < nb_datas; i++)
     {
-      g_ptr_array_add (gpa, fread_gstring (fh));
+      if (fh >= 0)
+	g_ptr_array_add (gpa, read_gstring (fh));
+      else
+	g_ptr_array_add (gpa, gzread_gstring (gzfh));
     }
   return (gpa);
 }
 
-static gint32
-load_compressed_image (char *filename)
-{
-  int pid;
-  int status;
-
-
-  if ((pid = fork ()) < 0)
-    {
-      g_message ("fork failed: %s\n", g_strerror (errno));
-      return -1;
-    }
-  else if (pid == 0)
-    {				/* child process */
-      FILE *f;
-      if (!(f = fopen ("/tmp/gtktalogtmpfile", "w+")))
-	{
-	  g_message ("gz: fopen failed: %s\n", g_strerror (errno));
-	  _exit (127);
-	}
-
-      if (-1 == dup2 (fileno (f), fileno (stdout)))
-	g_message ("gz: dup2 failed: %s\n", g_strerror (errno));
-
-      if (fnmatch ("*.gz", filename, 0) == 0)
-	execlp ("gzip", "gzip", "-cfd", filename, NULL);
-      if (fnmatch ("*.bz2", filename, 0) == 0)
-	execlp ("bzip2", "bzip2", "-cfd", filename, NULL);
-      if (fnmatch ("*.gz", filename, 0) == 0)
-	{
-	  g_message ("gzip: exec failed: %s\n", g_strerror (errno));
-	}
-      if (fnmatch ("*.bz2", filename, 0) == 0)
-	{
-	  g_message ("bzip: exec failed:  %s\n", g_strerror (errno));
-	}
-      _exit (127);
-    }
-  else
-    {				/* parent process */
-
-      waitpid (pid, &status, 0);
-
-      if (!WIFEXITED (status) || WEXITSTATUS (status) != 0)
-	{
-	  if (fnmatch ("*.gz", filename, 0) == 0)
-	    {
-	      g_message ("gz: gzip exited abnormally on file %s\n",
-			 "/tmp/gtktalogtmpfile");
-	    }
-	  if (fnmatch ("*.bz2", filename, 0) == 0)
-	    {
-	      g_message ("bz2: bzip2 exited abnormally on file %s\n",
-			 "/tmp/gtktalogtmpfile");
-	    }
-	  return -1;
-	}
-    }
-  return 0;
-}
-
 int
 load_cat_from_file (char *filename, FOLDER * racine)
 {
-  char fileformat[12];
+  char fileformat[MAX_STRING_LENGTH];
   gint l;
+  int catalog;
+  gzFile gzcatalog;
 
   if ((g_file_test (filename, G_FILE_TEST_ISFILE)) != TRUE)
     {
@@ -200,36 +191,16 @@ load_cat_from_file (char *filename, FOLDER * racine)
 		    main_window);
       return (-1);
     }
-  if ((fnmatch ("*.gz", filename, 0) == 0)
-      || (fnmatch ("*.bz2", filename, 0) == 0))
+  if ((catalog = open (filename, O_RDONLY)) == -1)
     {
-      if (load_compressed_image (filename) == 0)
-	{
-	  if ((catalog = fopen ("/tmp/gtktalogtmpfile", "r")) == NULL)
-	    {
-	      ERROR_DIALOG ("Can't open file.", main_window);
-	      return (-1);
-	    }
-	}
-      else
-	{
-	  ERROR_DIALOG ("Can't uncompress file.", main_window);
-	  return (-1);
-	}
-    }
-  else
-    {
-      if ((catalog = fopen (filename, "r")) == NULL)
-	{
-	  ERROR_DIALOG ("Can't open file.", main_window);
-	  return (-1);
-	}
+      ERROR_DIALOG ("Can't open file.", main_window);
+      return (-1);
     }
 
-  fread (&l, sizeof (gint), 1, catalog);
-  if (l != 10)
+  read (catalog, &l, sizeof (gint));
+  if ((l > 20) || (l < 5))
     {
-      fclose (catalog);
+      close (catalog);
 
       /* Just a test to detect an version 1 or 2 of the file format */
       if (strncmp ((gchar *) l, "gtktalog", sizeof (gint)) == 0)
@@ -245,19 +216,32 @@ load_cat_from_file (char *filename, FOLDER * racine)
 	}
       return (-1);
     }
-  fread (&fileformat, sizeof (gchar), l, catalog);
+  read (catalog, &fileformat, sizeof (gchar) * l);
   fileformat[l] = 0;
-  if (strcmp (fileformat, "gtktalog 3"))
+  if ((!strcmp (fileformat, "gtktalog 3"))
+      && (!strcmp (fileformat, "gtktalog 3gz")))
     {
-      fclose (catalog);
+      close (catalog);
       ERROR_DIALOG ("File is not a GTKtalog file (v3 file format).",
 		    main_window);
       return (-1);
     }
-  racine->descriptions = load_gptrarray_of_gstrings (catalog);
-  racine->categories = load_gptrarray_of_gstrings (catalog);
-  load_folder_tree (racine, catalog);
-  fclose (catalog);
+  if (strcmp (fileformat, "gtktalog 3"))
+    {
+      gzcatalog = gzdopen (catalog, "rb");
+      racine->descriptions = load_gptrarray_of_gstrings (gzcatalog, -1);
+      racine->categories = load_gptrarray_of_gstrings (gzcatalog, -1);
+      load_folder_tree (racine, gzcatalog, -1);
+      gzclose (gzcatalog);
+    }
+  else
+    {
+      racine->descriptions = load_gptrarray_of_gstrings (NULL, catalog);
+      racine->categories = load_gptrarray_of_gstrings (NULL, catalog);
+      load_folder_tree (racine, NULL, catalog);
+      close (catalog);
+
+    }
   racine->is_modified = FALSE;
   if (racine->catalog_filename)
     g_string_free (racine->catalog_filename, TRUE);
