@@ -17,13 +17,15 @@
  */
 #ifndef __LOADCAT_H__
 #define __LOADCAT_H__
+#include <zlib.h>
 
 #include "config_common.h"
 
-GString *fread_gstring(FILE *catalog);
-FILE_DATA *fread_filedata(FILE *catalog);
-void load_folder_tree(FOLDER *folder, FILE *catalog);
-GPtrArray *load_gptrarray_of_gstrings(FILE *catalog);
+GString *read_gstring(int fh);
+GString *gzread_gstring(gzFile fh);
+FILE_DATA *read_filedata(gzFile gzfh, int fh);
+void load_folder_tree(FOLDER *racine, gzFile gzcatalog, int catalog);
+GPtrArray *load_gptrarray_of_gstrings(gzFile gzfh, int fh);
 int load_cat_from_file(char *filename, FOLDER *racine);
 
 #endif
