@@ -179,10 +179,13 @@ main (int argc, char *argv[])
 
   if (_filename)
     {
+      gtk_entry_set_text (GTK_ENTRY (path_entry), "");
+      my_config->racine = init_folder ();
       load_cat_from_file (_filename->str, my_config->racine);
       g_node_traverse (my_config->racine->tree,
 		       G_PRE_ORDER, G_TRAVERSE_ALL, -1, add_gnode_to_ctree,
 		       my_config->racine);
+      ctree_window_add (my_config->racine->ctree);
       update_tree (my_config->racine);
 
       gtk_window_set_title (GTK_WINDOW (main_window),
