@@ -33,10 +33,35 @@
 #include "on.xpm"
 #include "off.xpm"
 
+/* setup dialog */
+static GtkWidget *include_list_box;
+static GtkWidget *mount_entry;
+static GtkWidget *umount_entry;
+static GtkWidget *mount_point_entry;
+static GtkWidget *eject_entry;
+static GtkWidget *checkbutton_dus;
+static GtkWidget *checkbutton_warnMount;
+static GtkWidget *checkbutton_eject;
+static GdkPixmap *on_pix;
+static GdkPixmap *off_pix;
+static GtkStyle *style;
+static GdkBitmap *state_mask;
+static GtkWidget *window;
+static gint selected_row;
+
+/* Add type dialog */
+static GtkWidget *include_type_entry;
+static GtkWidget *include_prog_entry;
+static GtkWidget *include_arg_entry;
+static GtkWidget *include_viewer_entry;
+static GtkWidget *include_viewer_arg_entry;
+static GtkWidget *add_type_window;
+
+EXTENSIONS *selected_extension;
+
 gboolean
 extensions_traverse (gpointer key, gpointer value, gpointer data)
 {
-  SETUP_WIDGETS *sw = data;
   EXTENSIONS *e = value;
   gint i;
   gchar *tmp[6] = { "", "", "", "", "", "" };
@@ -46,14 +71,14 @@ extensions_traverse (gpointer key, gpointer value, gpointer data)
   tmp[3] = e->arg->str;
   tmp[4] = e->viewer->str;
   tmp[5] = e->viewerarg->str;
-  i = gtk_clist_append ((GtkCList *) sw->include_list_box, (gchar **) tmp);
-  gtk_clist_set_row_data ((GtkCList *) sw->include_list_box, i, e);
+  i = gtk_clist_append ((GtkCList *) include_list_box, (gchar **) tmp);
+  gtk_clist_set_row_data ((GtkCList *) include_list_box, i, e);
   if (e->state == FALSE)
-    gtk_clist_set_pixmap ((GtkCList *) sw->include_list_box, i, 0,
-			  sw->off_pix, sw->state_mask);
+    gtk_clist_set_pixmap ((GtkCList *) include_list_box, i, 0,
+			  off_pix, state_mask);
   else
-    gtk_clist_set_pixmap ((GtkCList *) sw->include_list_box, i, 0,
-			  sw->on_pix, sw->state_mask);
+    gtk_clist_set_pixmap ((GtkCList *) include_list_box, i, 0,
+			  on_pix, state_mask);
   return (FALSE);
 }
 
@@ -89,22 +114,32 @@ setup_ok_clicked (GtkWidget * w, gpointer data)
 {
   FILE *f;
   GString *conf_file;
-  CONFIG *my_config = data;
-  SETUP_WIDGETS *sw = g_hash_table_lookup (my_config->stuff, "_setup: SETUP WIDGETS");
 
   g_string_free (my_config->mount, TRUE);
   g_string_free (my_config->umount, TRUE);
   g_string_free (my_config->eject_prog, TRUE);
   g_string_free (my_config->mount_point, TRUE);
 
+  /* Get Config */
   my_config->mount =
-    g_string_new (gtk_entry_get_text (GTK_ENTRY (sw->mount_entry)));
+    g_string_new (gtk_entry_get_text (GTK_ENTRY (mount_entry)));
   my_config->umount =
-    g_string_new (gtk_entry_get_text (GTK_ENTRY (sw->umount_entry)));
+    g_string_new (gtk_entry_get_text (GTK_ENTRY (umount_entry)));
   my_config->eject_prog =
-    g_string_new (gtk_entry_get_text (GTK_ENTRY (sw->eject_entry)));
+    g_string_new (gtk_entry_get_text (GTK_ENTRY (eject_entry)));
   my_config->mount_point =
-    g_string_new (gtk_entry_get_text (GTK_ENTRY (sw->mount_point_entry)));
+    g_string_new (gtk_entry_get_text (GTK_ENTRY (mount_point_entry)));
+  my_config->use_supermount = check_supermount(my_config);
+  /* Config Options */
+  my_config->use_du_s =
+    (gboolean) (gtk_toggle_button_get_active
+		(GTK_TOGGLE_BUTTON (checkbutton_dus)));
+  my_config->eject_disk =
+    (gboolean) (gtk_toggle_button_get_active
+		(GTK_TOGGLE_BUTTON (checkbutton_eject)));
+  my_config->warn_mount =
+    (gboolean) (gtk_toggle_button_get_active
+		(GTK_TOGGLE_BUTTON (checkbutton_warnMount)));
 
   conf_file = g_string_new (g_get_home_dir ());
   if (conf_file->len < 1)
@@ -113,7 +148,7 @@ setup_ok_clicked (GtkWidget * w, gpointer data)
 				  (gnome_warning_dialog_parented
 				   (_
 				    ("Home directory not found. Setup will not be saved."),
-				    GTK_WINDOW (my_config->window))));
+				    GTK_WINDOW (main_window))));
     }
   else
     {
@@ -126,7 +161,7 @@ setup_ok_clicked (GtkWidget * w, gpointer data)
 
       if (!(f = fopen (conf_file->str, "w+")))
 	{
-	  ERROR_DIALOG ("Can't save to config file", my_config->window);
+	  ERROR_DIALOG ("Can't save to config file", main_window);
 	}
       else
 	{
@@ -157,27 +192,33 @@ setup_ok_clicked (GtkWidget * w, gpointer data)
 	  fprintf (f, "EjectProgram %s\n", my_config->eject_prog->str);
 	  fprintf (f, _("# MountPoint path\n"));
 	  fprintf (f, "MountPoint %s\n", my_config->mount_point->str);
+
+	  /* Options */
+	  fprintf (f, _("\n# Options \n"));
+	  fprintf (f, _("# Only use True or False for the following\n"));
+	  fprintf (f, _("# Use du -s to get the size of the disk\n"));
+	  fprintf (f, "OptionDus %s\n",
+		   (my_config->use_du_s ? "True" : "False"));
+	  fprintf (f, _("# Warn if a disk is mounted\n"));
+	  fprintf (f, "OptionWarnMount %s\n",
+		   (my_config->warn_mount ? "True" : "False"));
+	  fprintf (f, _("# Eject the disk after a fast scan\n"));
+	  fprintf (f, "OptionEject %s\n",
+		   (my_config->eject_disk ? "True" : "False"));
+
 	  fclose (f);
 	}
 
     }
 
 
-  gtk_widget_destroy (sw->window);
-  g_free (sw);
-  g_hash_table_remove (my_config->stuff, "_setup: SETUP WIDGETS");
+  gtk_widget_destroy (window);
 }
 
 void
 add_type_ok_clicked (GtkWidget * w, gpointer data)
 {
-  CONFIG *my_config = data;
-  SETUP_WIDGETS *sw = g_hash_table_lookup (my_config->stuff, "_setup: SETUP WIDGETS");
-  EXTENSIONS *e = g_hash_table_lookup (my_config->stuff, "_setup: EXTENSIONS");
   EXTENSIONS *e2;
-  ADDTYPE_WIDGETS *aw =
-    g_hash_table_lookup (my_config->stuff, "_setup: ADDTYPE WIDGETS");
-
 
   char *temp_type;
   char *temp_prog;
@@ -186,101 +227,92 @@ add_type_ok_clicked (GtkWidget * w, gpointer data)
   char *temp_viewerarg;
 
   temp_type =
-    gtk_editable_get_chars (GTK_EDITABLE (aw->include_type_entry), 0, -1);
+    gtk_editable_get_chars (GTK_EDITABLE (include_type_entry), 0, -1);
   e2 = g_tree_lookup (my_config->extensions, temp_type);
 
-  if (e2 && (e2 != e))
+  if (e2 && (e2 != selected_extension))
     {
       gnome_dialog_run_and_close (GNOME_DIALOG
 				  (gnome_warning_dialog_parented
 				   (_
 				    ("This extension already exists. Nothing will be done."),
-				    GTK_WINDOW (aw->add_type_window))));
+				    GTK_WINDOW (add_type_window))));
       g_free (temp_type);
     }
   else
     {
-      if (sw->selected_row >= 0)
+      if (selected_row >= 0)
 	{
-	  g_string_free (e->extension, TRUE);
-	  g_string_free (e->prog, TRUE);
-	  g_string_free (e->arg, TRUE);
-	  g_string_free (e->viewer, TRUE);
-	  g_string_free (e->viewerarg, TRUE);
-	  g_tree_remove (my_config->extensions, e);
-	  gtk_clist_remove (GTK_CLIST (sw->include_list_box),
-			    sw->selected_row);
+	  g_string_free (selected_extension->extension, TRUE);
+	  g_string_free (selected_extension->prog, TRUE);
+	  g_string_free (selected_extension->arg, TRUE);
+	  g_string_free (selected_extension->viewer, TRUE);
+	  g_string_free (selected_extension->viewerarg, TRUE);
+	  g_tree_remove (my_config->extensions, selected_extension);
+	  gtk_clist_remove (GTK_CLIST (include_list_box),
+			    selected_row);
 
 	}
       else
 	{
 
-	  e = (EXTENSIONS *) g_malloc (sizeof (EXTENSIONS));
+	  selected_extension = (EXTENSIONS *) g_malloc (sizeof (EXTENSIONS));
 	}
       temp_type =
-	gtk_editable_get_chars (GTK_EDITABLE (aw->include_type_entry), 0, -1);
+	gtk_editable_get_chars (GTK_EDITABLE (include_type_entry), 0, -1);
       temp_prog =
-	gtk_editable_get_chars (GTK_EDITABLE (aw->include_prog_entry), 0, -1);
+	gtk_editable_get_chars (GTK_EDITABLE (include_prog_entry), 0, -1);
       temp_arg =
-	gtk_editable_get_chars (GTK_EDITABLE (aw->include_arg_entry), 0, -1);
+	gtk_editable_get_chars (GTK_EDITABLE (include_arg_entry), 0, -1);
       temp_viewer =
-	gtk_editable_get_chars (GTK_EDITABLE (aw->include_viewer_entry), 0,
+	gtk_editable_get_chars (GTK_EDITABLE (include_viewer_entry), 0,
 				-1);
       temp_viewerarg =
-	gtk_editable_get_chars (GTK_EDITABLE (aw->include_viewer_arg_entry),
+	gtk_editable_get_chars (GTK_EDITABLE (include_viewer_arg_entry),
 				0, -1);
-      e->extension = g_string_new (temp_type);
+      selected_extension->extension = g_string_new (temp_type);
       g_free (temp_type);
-      e->prog = g_string_new (temp_prog);
+      selected_extension->prog = g_string_new (temp_prog);
       g_free (temp_prog);
-      e->arg = g_string_new (temp_arg);
+      selected_extension->arg = g_string_new (temp_arg);
       g_free (temp_arg);
-      e->viewer = g_string_new (temp_viewer);
+      selected_extension->viewer = g_string_new (temp_viewer);
       g_free (temp_viewer);
-      e->viewerarg = g_string_new (temp_viewerarg);
+      selected_extension->viewerarg = g_string_new (temp_viewerarg);
       g_free (temp_viewerarg);
-      g_tree_insert (my_config->extensions, e->extension->str, e);
+      g_tree_insert (my_config->extensions, selected_extension->extension->str, selected_extension);
 
-      extensions_traverse (e->extension->str, e, sw);
+      extensions_traverse (selected_extension->extension->str, selected_extension, NULL);
 
     }
-  gtk_widget_destroy (aw->add_type_window);
-  g_free (aw);
-  g_hash_table_remove (my_config->stuff, "_setup: ADDTYPE WIDGETS");
+  gtk_widget_destroy (add_type_window);
 }
 
 
 void
 add_type (GtkWidget * w, gpointer data)
 {
-  CONFIG *my_config = data;
   GtkWidget *vbox, *hbox, *vbox2;
   GtkWidget *ok, *cancel;
   GtkWidget *frame;
   GtkWidget *label;
-  ADDTYPE_WIDGETS *aw;
-  EXTENSIONS *e = g_hash_table_lookup (my_config->stuff, "_setup: EXTENSIONS");
 
-  aw = (ADDTYPE_WIDGETS *) g_malloc (sizeof (ADDTYPE_WIDGETS));
-  g_hash_table_insert (my_config->stuff, "_setup: ADDTYPE WIDGETS", aw);
-
-
-  aw->add_type_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
-  gtk_window_set_position (GTK_WINDOW (aw->add_type_window),
+  add_type_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
+  gtk_window_set_position (GTK_WINDOW (add_type_window),
 			   GTK_WIN_POS_MOUSE);
-  gtk_widget_set_usize (GTK_WIDGET (aw->add_type_window), 300, 295);
-  gtk_window_set_title (GTK_WINDOW (aw->add_type_window), _("File type"));
-  gtk_window_set_policy (GTK_WINDOW (aw->add_type_window), FALSE, FALSE,
+  gtk_widget_set_usize (GTK_WIDGET (add_type_window), 300, 295);
+  gtk_window_set_title (GTK_WINDOW (add_type_window), _("File type"));
+  gtk_window_set_policy (GTK_WINDOW (add_type_window), FALSE, FALSE,
 			 TRUE);
-  gtk_container_border_width (GTK_CONTAINER (aw->add_type_window), 10);
-  gtk_signal_connect (GTK_OBJECT (aw->add_type_window), "delete_event",
+  gtk_container_border_width (GTK_CONTAINER (add_type_window), 10);
+  gtk_signal_connect (GTK_OBJECT (add_type_window), "delete_event",
 		      GTK_SIGNAL_FUNC (gtk_widget_destroy),
-		      GTK_OBJECT (aw->add_type_window));
+		      GTK_OBJECT (add_type_window));
 
-  gtk_grab_add (aw->add_type_window);
+  gtk_grab_add (add_type_window);
 
   vbox = gtk_vbox_new (FALSE, 0);
-  gtk_container_add (GTK_CONTAINER (aw->add_type_window), vbox);
+  gtk_container_add (GTK_CONTAINER (add_type_window), vbox);
   gtk_widget_show (vbox);
   /***************/
   frame = gtk_frame_new (NULL);
@@ -294,16 +326,16 @@ add_type (GtkWidget * w, gpointer data)
   gtk_container_add (GTK_CONTAINER (frame), vbox2);
   gtk_widget_show (vbox2);
 
-  aw->include_type_entry = gtk_entry_new_with_max_length (80);
+  include_type_entry = gtk_entry_new_with_max_length (80);
 
-  if (e)
-    gtk_entry_set_text (GTK_ENTRY (aw->include_type_entry),
-			e->extension->str);
-  gtk_entry_select_region (GTK_ENTRY (aw->include_type_entry), 0,
-			   GTK_ENTRY (aw->include_type_entry)->text_length);
-  gtk_box_pack_start (GTK_BOX (vbox2), aw->include_type_entry, FALSE, FALSE,
+  if (selected_extension)
+    gtk_entry_set_text (GTK_ENTRY (include_type_entry),
+			selected_extension->extension->str);
+  gtk_entry_select_region (GTK_ENTRY (include_type_entry), 0,
+			   GTK_ENTRY (include_type_entry)->text_length);
+  gtk_box_pack_start (GTK_BOX (vbox2), include_type_entry, FALSE, FALSE,
 		      0);
-  gtk_widget_show (aw->include_type_entry);
+  gtk_widget_show (include_type_entry);
 
   label = gtk_label_new (_("For example '.rpm'"));
   gtk_box_pack_start (GTK_BOX (vbox2), label, TRUE, FALSE, 0);
@@ -322,15 +354,15 @@ add_type (GtkWidget * w, gpointer data)
   gtk_container_add (GTK_CONTAINER (frame), vbox2);
   gtk_widget_show (vbox2);
 
-  aw->include_prog_entry = gtk_entry_new_with_max_length (80);
+  include_prog_entry = gtk_entry_new_with_max_length (80);
 
-  if (e)
-    gtk_entry_set_text (GTK_ENTRY (aw->include_prog_entry), e->prog->str);
-  gtk_entry_select_region (GTK_ENTRY (aw->include_prog_entry), 0,
-			   GTK_ENTRY (aw->include_prog_entry)->text_length);
-  gtk_box_pack_start (GTK_BOX (vbox2), aw->include_prog_entry, FALSE, FALSE,
+  if (selected_extension)
+    gtk_entry_set_text (GTK_ENTRY (include_prog_entry), selected_extension->prog->str);
+  gtk_entry_select_region (GTK_ENTRY (include_prog_entry), 0,
+			   GTK_ENTRY (include_prog_entry)->text_length);
+  gtk_box_pack_start (GTK_BOX (vbox2), include_prog_entry, FALSE, FALSE,
 		      0);
-  gtk_widget_show (aw->include_prog_entry);
+  gtk_widget_show (include_prog_entry);
 
   label = gtk_label_new (_("Leave blank to include all the file"));
   gtk_box_pack_start (GTK_BOX (vbox2), label, TRUE, FALSE, 0);
@@ -344,14 +376,14 @@ add_type (GtkWidget * w, gpointer data)
   gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_ETCHED_OUT);
   gtk_widget_show (frame);
 
-  aw->include_arg_entry = gtk_entry_new_with_max_length (80);
+  include_arg_entry = gtk_entry_new_with_max_length (80);
 
-  if (e)
-    gtk_entry_set_text (GTK_ENTRY (aw->include_arg_entry), e->arg->str);
-  gtk_entry_select_region (GTK_ENTRY (aw->include_arg_entry), 0,
-			   GTK_ENTRY (aw->include_arg_entry)->text_length);
-  gtk_container_add (GTK_CONTAINER (frame), aw->include_arg_entry);
-  gtk_widget_show (aw->include_arg_entry);
+  if (selected_extension)
+    gtk_entry_set_text (GTK_ENTRY (include_arg_entry), selected_extension->arg->str);
+  gtk_entry_select_region (GTK_ENTRY (include_arg_entry), 0,
+			   GTK_ENTRY (include_arg_entry)->text_length);
+  gtk_container_add (GTK_CONTAINER (frame), include_arg_entry);
+  gtk_widget_show (include_arg_entry);
 
   /***********/
 
@@ -366,15 +398,15 @@ add_type (GtkWidget * w, gpointer data)
   gtk_container_add (GTK_CONTAINER (frame), vbox2);
   gtk_widget_show (vbox2);
 
-  aw->include_viewer_entry = gtk_entry_new_with_max_length (80);
+  include_viewer_entry = gtk_entry_new_with_max_length (80);
 
-  if (e)
-    gtk_entry_set_text (GTK_ENTRY (aw->include_viewer_entry), e->viewer->str);
-  gtk_entry_select_region (GTK_ENTRY (aw->include_viewer_entry), 0,
-			   GTK_ENTRY (aw->include_viewer_entry)->text_length);
-  gtk_box_pack_start (GTK_BOX (vbox2), aw->include_viewer_entry, FALSE, FALSE,
+  if (selected_extension)
+    gtk_entry_set_text (GTK_ENTRY (include_viewer_entry), selected_extension->viewer->str);
+  gtk_entry_select_region (GTK_ENTRY (include_viewer_entry), 0,
+			   GTK_ENTRY (include_viewer_entry)->text_length);
+  gtk_box_pack_start (GTK_BOX (vbox2), include_viewer_entry, FALSE, FALSE,
 		      0);
-  gtk_widget_show (aw->include_viewer_entry);
+  gtk_widget_show (include_viewer_entry);
 
   label = gtk_label_new (_("Leave blank to use GTKtalog viewer"));
   gtk_box_pack_start (GTK_BOX (vbox2), label, TRUE, FALSE, 0);
@@ -394,17 +426,17 @@ add_type (GtkWidget * w, gpointer data)
   gtk_container_add (GTK_CONTAINER (frame), vbox2);
   gtk_widget_show (vbox2);
 
-  aw->include_viewer_arg_entry = gtk_entry_new_with_max_length (80);
+  include_viewer_arg_entry = gtk_entry_new_with_max_length (80);
 
-  if (e)
-    gtk_entry_set_text (GTK_ENTRY (aw->include_viewer_arg_entry),
-			e->viewerarg->str);
-  gtk_entry_select_region (GTK_ENTRY (aw->include_viewer_arg_entry), 0,
-			   GTK_ENTRY (aw->
+  if (selected_extension)
+    gtk_entry_set_text (GTK_ENTRY (include_viewer_arg_entry),
+			selected_extension->viewerarg->str);
+  gtk_entry_select_region (GTK_ENTRY (include_viewer_arg_entry), 0,
+			   GTK_ENTRY (
 				      include_viewer_arg_entry)->text_length);
-  gtk_box_pack_start (GTK_BOX (vbox2), aw->include_viewer_arg_entry, FALSE,
+  gtk_box_pack_start (GTK_BOX (vbox2), include_viewer_arg_entry, FALSE,
 		      FALSE, 0);
-  gtk_widget_show (aw->include_viewer_arg_entry);
+  gtk_widget_show (include_viewer_arg_entry);
 
 
 /********************************/
@@ -416,10 +448,10 @@ add_type (GtkWidget * w, gpointer data)
 
   ok = gtk_button_new_with_label (_("Ok"));
   GTK_WIDGET_SET_FLAGS (ok, GTK_CAN_DEFAULT);
-  gtk_window_set_default (GTK_WINDOW (aw->add_type_window), ok);
+  gtk_window_set_default (GTK_WINDOW (add_type_window), ok);
   gtk_box_pack_start (GTK_BOX (hbox), ok, TRUE, TRUE, 0);
   gtk_signal_connect (GTK_OBJECT (ok), "clicked",
-		      GTK_SIGNAL_FUNC (add_type_ok_clicked), my_config);
+		      GTK_SIGNAL_FUNC (add_type_ok_clicked), NULL);
   gtk_widget_show (ok);
 
   cancel = gtk_button_new_with_label (_("Cancel"));
@@ -427,13 +459,13 @@ add_type (GtkWidget * w, gpointer data)
   gtk_box_pack_start (GTK_BOX (hbox), cancel, TRUE, TRUE, 0);
   gtk_signal_connect_object (GTK_OBJECT (cancel), "clicked",
 			     GTK_SIGNAL_FUNC (gtk_widget_destroy),
-			     GTK_OBJECT (aw->add_type_window));
+			     GTK_OBJECT (add_type_window));
   gtk_widget_show (cancel);
 
   gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
   gtk_widget_show (hbox);
 
-  gtk_widget_show (aw->add_type_window);
+  gtk_widget_show (add_type_window);
 
 }
 
@@ -443,27 +475,25 @@ void
 include_selection_made (GtkWidget * clist, gint row, gint column,
 			GdkEventButton * event, gpointer data)
 {
-  CONFIG *my_config = data;
-  SETUP_WIDGETS *sw = g_hash_table_lookup (my_config->stuff, "_setup: SETUP WIDGETS");
   EXTENSIONS *clrow_data;
 
-  sw->selected_row = row;
-  clrow_data = gtk_clist_get_row_data (GTK_CLIST (sw->include_list_box), row);
-  g_hash_table_insert (my_config->stuff, "_setup: EXTENSIONS", clrow_data);
+  selected_row = row;
+  clrow_data = gtk_clist_get_row_data (GTK_CLIST (include_list_box), row);
+  selected_extension = clrow_data;
 
   if (event != NULL && event->button == 1 && event->type == GDK_2BUTTON_PRESS)
     {
       if (clrow_data->state == FALSE)
 	{
 	  clrow_data->state = TRUE;
-	  gtk_clist_set_pixmap ((GtkCList *) sw->include_list_box, row, 0,
-				sw->on_pix, sw->state_mask);
+	  gtk_clist_set_pixmap ((GtkCList *) include_list_box, row, 0,
+				on_pix, state_mask);
 	}
       else
 	{
 	  clrow_data->state = FALSE;
-	  gtk_clist_set_pixmap ((GtkCList *) sw->include_list_box, row, 0,
-				sw->off_pix, sw->state_mask);
+	  gtk_clist_set_pixmap ((GtkCList *) include_list_box, row, 0,
+				off_pix, state_mask);
 	}
     }
 }
@@ -472,47 +502,36 @@ void
 include_unselection_made (GtkWidget * clist, gint row, gint column,
 			  GdkEventButton * event, gpointer data)
 {
-  CONFIG *my_config = data;
-  SETUP_WIDGETS *sw = g_hash_table_lookup (my_config->stuff, "_setup: SETUP WIDGETS");
-  g_hash_table_remove (my_config->stuff, "_setup: EXTENSIONS");
-  sw->selected_row = -1;
+  selected_extension = NULL;
+  selected_row = -1;
 }
 
 void
 include_edit_button_clicked (GtkWidget * w, gpointer data)
 {
-  CONFIG *my_config = data;
-  SETUP_WIDGETS *sw = g_hash_table_lookup (my_config->stuff, "_setup: SETUP WIDGETS");
-  if (g_hash_table_lookup (my_config->stuff, "_setup: EXTENSIONS")
-      && (sw->selected_row >= 0))
-    add_type (w, my_config);
+  if (selected_extension
+      && (selected_row >= 0))
+    add_type (w, NULL);
 }
 
 
 void
 include_delete_button_clicked (GtkWidget * w, gpointer data)
 {
-  CONFIG *my_config = data;
-  SETUP_WIDGETS *sw = g_hash_table_lookup (my_config->stuff, "_setup: SETUP WIDGETS");
-
-  if (sw->selected_row >= 0)
+  if (selected_row >= 0)
     {
       g_tree_remove (my_config->extensions,
-		     g_hash_table_lookup (my_config->stuff, "_setup: EXTENSIONS"));
-      g_free (g_hash_table_lookup (my_config->stuff, "_setup: EXTENSIONS"));
-      g_hash_table_remove (my_config->stuff, "_setup: EXTENSIONS");
-      gtk_clist_remove (GTK_CLIST (sw->include_list_box), sw->selected_row);
-      sw->selected_row = -1;
+		           selected_extension );
+      g_free (selected_extension);
+      selected_extension = NULL;
+      gtk_clist_remove (GTK_CLIST (include_list_box), selected_row);
+      selected_row = -1;
     }
 }
 static void
 cancel_clicked (GtkWidget * w, gpointer data)
 {
-  CONFIG *my_config = data;
-  SETUP_WIDGETS *sw = g_hash_table_lookup (my_config->stuff, "_setup: SETUP WIDGETS");
-  gtk_widget_destroy (sw->window);
-  g_free (sw);
-  g_hash_table_remove (my_config->stuff, "_setup: SETUP WIDGETS");
+  gtk_widget_destroy (window);
   return;
 }
 
@@ -522,7 +541,7 @@ static void destroy_window(GtkWidget * w, GdkEventKey *event, gpointer data)
 }
 
 void
-run_setup (GtkWidget * w, gpointer data)
+run_setup ()
 {
   GtkWidget *ok;
   GtkWidget *cancel;
@@ -536,9 +555,8 @@ run_setup (GtkWidget * w, gpointer data)
   GtkWidget *add_button;
   GtkWidget *delete_button;
   GtkWidget *edit_button;
-  GtkWidget *vbox2;
+  GtkWidget *vbox2, *vbox3;
   GtkWidget *scrolled_window;
-  SETUP_WIDGETS *setup_widgets;
   gchar *titles[6] = { N_("State"),
     N_("File type"),
     N_("Plugin"),
@@ -547,51 +565,47 @@ run_setup (GtkWidget * w, gpointer data)
     N_("Viewer Options")
   };
 
-  CONFIG *my_config = data;
+  selected_row = -1;
+  selected_extension = NULL;
 
-  setup_widgets = (SETUP_WIDGETS *) g_malloc (sizeof (SETUP_WIDGETS));
-  g_hash_table_insert (my_config->stuff, "_setup: SETUP WIDGETS", setup_widgets);
-  setup_widgets->selected_row = -1;
-
-
-  setup_widgets->window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
-  gtk_window_set_policy (GTK_WINDOW (setup_widgets->window), FALSE, FALSE,
+  window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
+  gtk_window_set_policy (GTK_WINDOW (window), FALSE, FALSE,
 			 TRUE);
-  gtk_window_set_title (GTK_WINDOW (setup_widgets->window), _("Setup"));
-  gtk_widget_set_usize (GTK_WIDGET (setup_widgets->window), 370, 335);
-  gtk_signal_connect (GTK_OBJECT (setup_widgets->window),
-		      "delete_event", destroy_window, my_config);
+  gtk_window_set_title (GTK_WINDOW (window), _("Setup"));
+  gtk_widget_set_usize (GTK_WIDGET (window), 370, 335);
+  gtk_signal_connect (GTK_OBJECT (window),
+		      "delete_event", destroy_window, NULL);
 
 
-  gtk_widget_realize (setup_widgets->window);
+  gtk_widget_realize (window);
 
   /* pixmap stuff */
-  setup_widgets->style = gtk_widget_get_style (setup_widgets->window);
-  setup_widgets->on_pix =
-    gdk_pixmap_create_from_xpm_d (setup_widgets->window->window,
-				  &(setup_widgets->state_mask),
-				  &(setup_widgets->
+  style = gtk_widget_get_style (window);
+  on_pix =
+    gdk_pixmap_create_from_xpm_d (window->window,
+				  &(state_mask),
+				  &(
 				    style)->bg[GTK_STATE_NORMAL],
 				  (gchar **) on_xpm);
-  gtk_widget_unrealize (setup_widgets->window);
+  gtk_widget_unrealize (window);
 
-  gtk_widget_realize (setup_widgets->window);
-  setup_widgets->style = gtk_widget_get_style (setup_widgets->window);
-  setup_widgets->off_pix =
-    gdk_pixmap_create_from_xpm_d (setup_widgets->window->window,
-				  &(setup_widgets->state_mask),
-				  &(setup_widgets->
+  gtk_widget_realize (window);
+  style = gtk_widget_get_style (window);
+  off_pix =
+    gdk_pixmap_create_from_xpm_d (window->window,
+				  &(state_mask),
+				  &(
 				    style)->bg[GTK_STATE_NORMAL],
 				  (gchar **) off_xpm);
-  gtk_widget_unrealize (setup_widgets->window);
+  gtk_widget_unrealize (window);
 
-  gtk_container_set_border_width (GTK_CONTAINER (setup_widgets->window), 2);
-  gtk_grab_add (setup_widgets->window);
+  gtk_container_set_border_width (GTK_CONTAINER (window), 2);
+  gtk_grab_add (window);
 
 
   /* The notebook */
   setup_vbox = gtk_vbox_new (FALSE, 4);
-  gtk_container_add (GTK_CONTAINER (setup_widgets->window), setup_vbox);
+  gtk_container_add (GTK_CONTAINER (window), setup_vbox);
   gtk_widget_show (setup_vbox);
 
   notebook = gtk_notebook_new ();
@@ -624,42 +638,42 @@ run_setup (GtkWidget * w, gpointer data)
   gtk_widget_show (scrolled_window);
 
   /* setup box to associate programs to file extensions */
-  setup_widgets->include_list_box = gtk_clist_new_with_titles (6, titles);
-  gtk_signal_connect (GTK_OBJECT (setup_widgets->include_list_box),
+  include_list_box = gtk_clist_new_with_titles (6, titles);
+  gtk_signal_connect (GTK_OBJECT (include_list_box),
 		      "select_row", GTK_SIGNAL_FUNC (include_selection_made),
-		      my_config);
-  gtk_signal_connect (GTK_OBJECT (setup_widgets->include_list_box),
+		      NULL);
+  gtk_signal_connect (GTK_OBJECT (include_list_box),
 		      "unselect_row",
-		      GTK_SIGNAL_FUNC (include_unselection_made), my_config);
+		      GTK_SIGNAL_FUNC (include_unselection_made), NULL);
 
-  gtk_clist_set_shadow_type (GTK_CLIST (setup_widgets->include_list_box),
+  gtk_clist_set_shadow_type (GTK_CLIST (include_list_box),
 			     GTK_SHADOW_OUT);
-  gtk_clist_set_column_width (GTK_CLIST (setup_widgets->include_list_box), 0,
+  gtk_clist_set_column_width (GTK_CLIST (include_list_box), 0,
 			      25);
-  gtk_clist_set_column_width (GTK_CLIST (setup_widgets->include_list_box), 1,
+  gtk_clist_set_column_width (GTK_CLIST (include_list_box), 1,
 			      60);
-  gtk_clist_set_column_width (GTK_CLIST (setup_widgets->include_list_box), 2,
+  gtk_clist_set_column_width (GTK_CLIST (include_list_box), 2,
 			      60);
-  gtk_clist_set_column_width (GTK_CLIST (setup_widgets->include_list_box), 3,
+  gtk_clist_set_column_width (GTK_CLIST (include_list_box), 3,
 			      60);
-  gtk_clist_set_column_width (GTK_CLIST (setup_widgets->include_list_box), 4,
+  gtk_clist_set_column_width (GTK_CLIST (include_list_box), 4,
 			      60);
   // gtk_signal_connect(GTK_OBJECT(clist), "button_press_event", GTK_SIGNAL_FUNC(clist_pressed), NULL);
   gtk_clist_set_column_justification (GTK_CLIST
-				      (setup_widgets->include_list_box), 0,
+				      (include_list_box), 0,
 				      GTK_JUSTIFY_CENTER);
   gtk_clist_set_column_auto_resize (GTK_CLIST
-				    (setup_widgets->include_list_box), 5,
+				    (include_list_box), 5,
 				    TRUE);
 
   /* fill the clist */
   g_tree_traverse (my_config->extensions, extensions_traverse, G_PRE_ORDER,
-		   setup_widgets);
+		   NULL);
 
   /* show the clist */
   gtk_container_add (GTK_CONTAINER (scrolled_window),
-		     setup_widgets->include_list_box);
-  gtk_widget_show (setup_widgets->include_list_box);
+		     include_list_box);
+  gtk_widget_show (include_list_box);
 
   hbox = gtk_hbutton_box_new ();
   gtk_button_box_set_layout (GTK_BUTTON_BOX (hbox), GTK_BUTTONBOX_END);
@@ -669,7 +683,7 @@ run_setup (GtkWidget * w, gpointer data)
   GTK_WIDGET_SET_FLAGS (add_button, GTK_CAN_DEFAULT);
   gtk_box_pack_start (GTK_BOX (hbox), add_button, FALSE, FALSE, 0);
   gtk_signal_connect (GTK_OBJECT (add_button), "clicked",
-		      GTK_SIGNAL_FUNC (add_type), my_config);
+		      GTK_SIGNAL_FUNC (add_type), NULL);
   gtk_widget_show (add_button);
 
   edit_button = gtk_button_new_with_label (_("Edit"));
@@ -677,7 +691,7 @@ run_setup (GtkWidget * w, gpointer data)
   gtk_box_pack_start (GTK_BOX (hbox), edit_button, FALSE, FALSE, 0);
   gtk_signal_connect (GTK_OBJECT (edit_button), "clicked",
 		      GTK_SIGNAL_FUNC (include_edit_button_clicked),
-		      my_config);
+		      NULL);
   gtk_widget_show (edit_button);
 
   delete_button = gtk_button_new_with_label (_("Delete"));
@@ -685,7 +699,7 @@ run_setup (GtkWidget * w, gpointer data)
   gtk_box_pack_start (GTK_BOX (hbox), delete_button, FALSE, FALSE, 0);
   gtk_signal_connect (GTK_OBJECT (delete_button), "clicked",
 		      GTK_SIGNAL_FUNC (include_delete_button_clicked),
-		      my_config);
+		      NULL);
   gtk_widget_show (delete_button);
 
   gtk_box_pack_start (GTK_BOX (main_vbox), hbox, FALSE, TRUE, 0);
@@ -720,12 +734,12 @@ run_setup (GtkWidget * w, gpointer data)
   gtk_container_add (GTK_CONTAINER (frame), vbox2);
   gtk_widget_show (vbox2);
 
-  setup_widgets->mount_entry = gtk_entry_new_with_max_length (200);
-  gtk_entry_set_text (GTK_ENTRY (setup_widgets->mount_entry),
+  mount_entry = gtk_entry_new_with_max_length (200);
+  gtk_entry_set_text (GTK_ENTRY (mount_entry),
 		      my_config->mount->str);
-  gtk_box_pack_start (GTK_BOX (vbox2), setup_widgets->mount_entry, FALSE,
+  gtk_box_pack_start (GTK_BOX (vbox2), mount_entry, FALSE,
 		      FALSE, 0);
-  gtk_widget_show (setup_widgets->mount_entry);
+  gtk_widget_show (mount_entry);
 
 	/*************/
 
@@ -740,12 +754,12 @@ run_setup (GtkWidget * w, gpointer data)
   gtk_container_add (GTK_CONTAINER (frame), vbox2);
   gtk_widget_show (vbox2);
 
-  setup_widgets->umount_entry = gtk_entry_new_with_max_length (200);
-  gtk_entry_set_text (GTK_ENTRY (setup_widgets->umount_entry),
+  umount_entry = gtk_entry_new_with_max_length (200);
+  gtk_entry_set_text (GTK_ENTRY (umount_entry),
 		      my_config->umount->str);
-  gtk_box_pack_start (GTK_BOX (vbox2), setup_widgets->umount_entry, FALSE,
+  gtk_box_pack_start (GTK_BOX (vbox2), umount_entry, FALSE,
 		      FALSE, 0);
-  gtk_widget_show (setup_widgets->umount_entry);
+  gtk_widget_show (umount_entry);
 
 	/**************/
 
@@ -760,12 +774,12 @@ run_setup (GtkWidget * w, gpointer data)
   gtk_container_add (GTK_CONTAINER (frame), vbox2);
   gtk_widget_show (vbox2);
 
-  setup_widgets->mount_point_entry = gtk_entry_new_with_max_length (200);
-  gtk_entry_set_text (GTK_ENTRY (setup_widgets->mount_point_entry),
+  mount_point_entry = gtk_entry_new_with_max_length (200);
+  gtk_entry_set_text (GTK_ENTRY (mount_point_entry),
 		      my_config->mount_point->str);
-  gtk_box_pack_start (GTK_BOX (vbox2), setup_widgets->mount_point_entry,
+  gtk_box_pack_start (GTK_BOX (vbox2), mount_point_entry,
 		      FALSE, FALSE, 0);
-  gtk_widget_show (setup_widgets->mount_point_entry);
+  gtk_widget_show (mount_point_entry);
 
   label =
     gtk_label_new (_
@@ -786,12 +800,53 @@ run_setup (GtkWidget * w, gpointer data)
   gtk_container_add (GTK_CONTAINER (frame), vbox2);
   gtk_widget_show (vbox2);
 
-  setup_widgets->eject_entry = gtk_entry_new_with_max_length (200);
-  gtk_entry_set_text (GTK_ENTRY (setup_widgets->eject_entry),
-		      my_config->eject_prog->str);
-  gtk_box_pack_start (GTK_BOX (vbox2), setup_widgets->eject_entry, FALSE,
-		      FALSE, 0);
-  gtk_widget_show (setup_widgets->eject_entry);
+  eject_entry = gtk_entry_new_with_max_length (200);
+  gtk_entry_set_text (GTK_ENTRY (eject_entry), my_config->eject_prog->str);
+  gtk_box_pack_start (GTK_BOX (vbox2), eject_entry, FALSE, FALSE, 0);
+  gtk_widget_show (eject_entry);
+
+/*****  General Options  *******************************************************************/
+
+  /* Create the frame */
+  frame = gtk_frame_new (_("General Options"));
+  gtk_container_set_border_width (GTK_CONTAINER (frame), 3);
+
+  gtk_widget_show (frame);
+
+  /* Create the top label */
+  label = gtk_label_new (_("Options"));
+  gtk_notebook_append_page (GTK_NOTEBOOK (notebook), frame, label);
+
+  /* VBox to keep the objects */
+  vbox3 = gtk_vbox_new (FALSE, 0);
+  gtk_container_add (GTK_CONTAINER (frame), vbox3);
+  gtk_widget_show (vbox3);
+
+  /* Option to use du -s or statfs */
+  /* FIXME : Find another label */
+  checkbutton_dus =
+    gtk_check_button_new_with_label (_("Use du -s instead of statfs"));
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (checkbutton_dus),
+				my_config->use_du_s);
+  gtk_widget_show (checkbutton_dus);
+  gtk_box_pack_start (GTK_BOX (vbox3), checkbutton_dus, FALSE, FALSE, 0);
+
+  /* Option to warn if the cd is mounted */
+  checkbutton_warnMount =
+    gtk_check_button_new_with_label (_("Warn if the CD is mounted"));
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (checkbutton_warnMount),
+				my_config->warn_mount);
+  gtk_widget_show (checkbutton_warnMount);
+  gtk_box_pack_start (GTK_BOX (vbox3), checkbutton_warnMount, FALSE, FALSE,
+		      0);
+
+  /* Option to eject the CD */
+  checkbutton_eject =
+    gtk_check_button_new_with_label (_("Eject the CD after fast scan"));
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (checkbutton_eject),
+				my_config->eject_disk);
+  gtk_widget_show (checkbutton_eject);
+  gtk_box_pack_start (GTK_BOX (vbox3), checkbutton_eject, FALSE, FALSE, 0);
 
 
 /*******************************************************************************************/
@@ -803,20 +858,20 @@ run_setup (GtkWidget * w, gpointer data)
   gtk_widget_show (bbox);
 
   ok = gtk_button_new_with_label (_("Save"));
-  gtk_object_set_user_data (GTK_OBJECT (ok), setup_widgets->window);
+  gtk_object_set_user_data (GTK_OBJECT (ok), window);
   GTK_WIDGET_SET_FLAGS (ok, GTK_CAN_DEFAULT);
-  gtk_window_set_default (GTK_WINDOW (setup_widgets->window), ok);
+  gtk_window_set_default (GTK_WINDOW (window), ok);
   gtk_box_pack_start (GTK_BOX (bbox), ok, TRUE, TRUE, 0);
   gtk_signal_connect (GTK_OBJECT (ok), "clicked",
-		      GTK_SIGNAL_FUNC (setup_ok_clicked), my_config);
+		      GTK_SIGNAL_FUNC (setup_ok_clicked), NULL);
   gtk_widget_show (ok);
 
   cancel = gtk_button_new_with_label (_("Cancel"));
   GTK_WIDGET_SET_FLAGS (cancel, GTK_CAN_DEFAULT);
   gtk_box_pack_start (GTK_BOX (bbox), cancel, TRUE, TRUE, 0);
   gtk_signal_connect (GTK_OBJECT (cancel), "clicked", cancel_clicked,
-		      my_config);
+		      NULL);
   gtk_widget_show (cancel);
 
-  gtk_widget_show (setup_widgets->window);
+  gtk_widget_show (window);
 }
